<?php

namespace Ems\portalGestor\Service;

use Doctrine\ORM\EntityManager;
use Ems\portalGestor\Util\DateUtil;
use Ems\portalGestor\Service\EstruturaService;
use Ems\portalGestor\Repository\DemonstrativoRepository;
use Ems\portalGestor\Repository\EscalaRepository;
use Ems\portalGestor\Util\Util;
use Ems\portalGestor\Util\ArrayUtil;
use Ems\portalGestor\Util\HcmServices;

class DemonstrativoService {
    /** @var DemonstrativoRepository */
    protected $demonstrativoRepository;

    /** @var EntityManager */
    protected $entityManager;

    /** @var HcmServices */
    protected $hcmServices;

    /** @var EscalaRepository */
    protected $escalaRepository;

    protected const messages = array(
        'ERRO_PERIODO_NAO_ENCONTRADO' => 'Período não encontrado.',
        'ERRO_IMPOSSIVEL_ENCONTRAR_PERIODO' => 'Não foi possível encontrar o período de apuração para o tipo do vínculo. Verifique se existe um período corresponte ao tipo do vínculo em questão e tente novamente.',
        'ERRO_VINCULO_NAO_ENCONTRADO' => 'Vínculo não encontrado.'
    );

    public function __construct(
        DemonstrativoRepository $demonstrativoRepository,
        EntityManager $entityManager,
        HcmServices $hcmServices,
        EstruturaService $estruturaService,
        EscalaRepository $escalaRepository
    ) {
        $this->demonstrativoRepository = $demonstrativoRepository;
        $this->entityManager = $entityManager;
        $this->hcmServices = $hcmServices;
        $this->estruturaService = $estruturaService;

        $this->periodoRepository = $this->entityManager->getRepository(
            "Teknisa\Model\Gestaopessoas\Entities\PtoPeriodoapuracao"
        );
        $this->vinculoRepository = $this->entityManager->getRepository(
            "Teknisa\Model\Gestaopessoas\Entities\GpeVinculom"
        );

        $this->hcmServices = $hcmServices;
        $this->escalaRepository = $escalaRepository;
    }

    public function getDiasDemonstrativo(array $row): array
    {
        $nrorg = $row['NRORG'];
        $nrvinculom = $row['NRVINCULOM'];
        $nrperiodoapuracao = $row['NRPERIODOAPURACAO'];

        $periodo = $this->getPeriodoPorTipoVinculo(
            $nrorg,
            $nrvinculom,
            $nrperiodoapuracao
        );

        $inicioPeriodo = $periodo['DTINICIALAPURACAO'];
        $fimPeriodo = $periodo['DTFINALAPURACAO'];

        $diasObjectPeriodo = DateUtil::getAllDaysBetweenDates($inicioPeriodo, $fimPeriodo);

        $diasPeriodo = array_map(function ($diaObject) {
            return DateUtil::dateObjectToString($diaObject);
        }, $diasObjectPeriodo);

        $diasDemonstrativo = [];

        foreach ($diasPeriodo as $dia) {
            $params = [
                'P_NRVINCULOM'        => $nrvinculom,
                'P_DTMESCOMPETENCIA'  => $inicioPeriodo,
                'P_NRPERIODOAPURACAO' => $periodo['NRPERIODOAPURACAO'],
                'P_NRORG'             => $nrorg,
                'P_DTAPURACAOPONTO'   => $dia
            ];

            $demonstrativo = $this->demonstrativoRepository->getDiaDemonstrativo($params);
            $demonstrativo["DIA"] = $dia;

            $rowMarcacao = [
                'NRVINCULOM'        => $row['NRVINCULOM'],
                'DTAPURACAOPONTO'   => $dia,
                'NRORG'             => $row['NRORG'],
            ];

            $ocorrencia = $this->getOcorrenciaDoDia($rowMarcacao);
            $demonstrativo = array_merge($demonstrativo, $ocorrencia);

            $entradaSaidaIds= $this->getDateIdEntradaSaida(
                $nrorg, $nrvinculom, $dia
            );

            $demonstrativo = array_merge($demonstrativo, $entradaSaidaIds);

            array_push($diasDemonstrativo, $demonstrativo);
        }

        $this->addIsDescandoProp($nrorg, $nrvinculom, $diasDemonstrativo);

        return $diasDemonstrativo;
    }

    public function getDateIdEntradaSaida(
        int $nrOrg,
        int $nrVinculo,
        string $date
    ): array {
        $entradaSaidaIds = [
            'IDDIA_ENTRADA' => null,
            'IDDIA_SAIDA'   => null
        ];

        $params = [
            'P_NRORG'     => $nrOrg,
            'P_NRVINCULO' => $nrVinculo,
            'P_DATE'      => $date
        ];

        $dateIds = $this->demonstrativoRepository->
            getAllDateIds($params);

        $lastIndex = count($dateIds) - 1;

        if (isset($dateIds[0])) {
            $entradaSaidaIds['IDDIA_ENTRADA'] = $dateIds[0]['IDDIA'];
        }

        if (isset($dateIds[$lastIndex])) {
            $entradaSaidaIds['IDDIA_SAIDA'] = $dateIds[$lastIndex]['IDDIA'];
        }

        return $entradaSaidaIds;
    }

    public function getPeriodosDemonstrativo(array $row): array
    {
        $params = [
            'P_NRORG'          => $row['NRORG']
        ];

        $periodos = $this->demonstrativoRepository->getPeriodosDemonstrativo($params);

        return $periodos;
    }

    public function getPeriodoDemonstratrivo(array $row): array
    {

        $params = [
            'P_NRPERIODOAPURACAO' => $row['NRPERIODOAPURACAO'],
            'P_NRORG'          => $row['NRORG']
        ];

        $periodos = $this->demonstrativoRepository->getPeriodoDemonstrativo($params);

        return $periodos;
    }

    public function getVinculosDoGestor(
        int $nrOrg,
        int $nrEstrutura,
        int $nrPeriodo,
        int $nrTipoVinculo
    ): array {

        $getVinculosDoGestorRoute = 'getVinculosGestor';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRESTRUTURAM'      => $nrEstrutura,
                'NRPERIODOAPURACAO' => $nrPeriodo,
                'NRTIPOVINCULOM'     => $nrTipoVinculo
            ]
        ];

        $result = $this->hcmServices->send(
            $getVinculosDoGestorRoute,
            $params,
            $nrOrg
        );

        if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }

        return $result['dataset']['getVinculosGestor'];
    }

    public function getMotivosOcorrencia($row): array
    {
        $params = [
            'P_NRORG' => $row['NRORG']
        ];

        $motivos = $this->demonstrativoRepository->getMotivosOcorrencia($params);

        return $motivos;
    }

    public function getMarcacoesDoDia($row) {
        $params = [
            // "P_NRORG"           => 20,
            "P_NRORG"           => $row['NRORG'],
            "P_NRVINCULOM"      => $row["NRVINCULOM"],
            "P_DTAPURACAOPONTO" => $row["DTAPURACAOPONTO"]
        ];

        $marcacoes = $this->demonstrativoRepository->getMarcacoesDoDia($params);


        return $marcacoes;
    }

    public function getOcorrenciaDoDia($row) {
        $params = [
            'P_NRORG' => $row['NRORG'],
            'P_NRVINCULOM' => $row["NRVINCULOM"],
            'P_DTAPURACAOPONTO' => $row['DTAPURACAOPONTO']
        ];

        $ocorrencias = $this->demonstrativoRepository->getOcorrenciasDoDia($params);

        $primeiraOcorrencia = Util::getIfSet($ocorrencias, '0') ?? [];

        return $primeiraOcorrencia;
    }

    public function retrocederApuracaoPeriodo(object $row) : string {
        $nrOrg = $row['NRORG'];
        $nrVinculo = $row['NRVINCULOM'];
        $nrPeriodoApuracao = $row['NRPERIODOAPURACAO'];

        $nrPeriodoApuracaoValidado = $this->getNrPeriodoApuracaoEstruturaPorTipoVinculo(
            $nrOrg,
            $nrVinculo,
            $nrPeriodoApuracao
        );

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRORG'             => $nrOrg,
                'NRVINCULOM'        => $nrVinculo,
                'NRPERIODOAPURACAO' => $nrPeriodoApuracaoValidado,
            ]
        ];

        $retrocesso = $this->hcmServices->send('retrocederApuracao', $params, $nrOrg);

        return $retrocesso['dataset']['retrocederApuracao']['deletedRows'];
    }

    public function apurarPeriodo(
        int $nrOrg,
        int $nrVinculo,
        int $nrPeriodo
    ) : bool {
        $nrPeriodoApuracaoValidado = $this->getNrPeriodoApuracaoEstruturaPorTipoVinculo(
            $nrOrg,
            $nrVinculo,
            $nrPeriodo
        );

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'NRPERIODOAPURACAO' => $nrPeriodoApuracaoValidado
            ]
        ];

        $apuracao = $this->hcmServices->send('apurarPeriodo', $params, $nrOrg);
        $apurado = $apuracao['dataset']['apurarPeriodo']['apurado'];

        return $apurado;
    }

    public function getParesMarcacoesDia(
        int $nrOrg,
        int $nrVinculo,
        string $dia
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
                'DIA'       => $dia
            ]
        ];

        $result = $this->hcmServices->send('getParesMarcacoesDia', $params, $nrOrg);
        $paresMarcacoes = $result['dataset']['getParesMarcacoesDia'];

        return $paresMarcacoes;
    }

    public function removeParMarcacao(
        int $nrOrg,
        int $nrVinculo,
        int $nrMarcacaoEntrada,
        int $nrMarcacaoSaida
    ) : int {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'NRMARCACAOENTRADA' => $nrMarcacaoEntrada,
                'NRMARCACAOSAIDA'   => $nrMarcacaoSaida
            ]
        ];

        $result = $this->hcmServices->send('removeParMarcacao', $params, $nrOrg);
        $marcacoesRemovidas = $result['dataset']['removeParMarcacao']['marcacoesRemovidas'];

        return $marcacoesRemovidas;
    }

    public function editParMarcacao(
        int $nrOrg,
        int $nrVinculo,
        int $nrMarcacaoEntrada,
        int $nrMarcacaoSaida,
        string $dia,
        string $hrEntrada,
        string $hrSaida,
        string $idDiaInicio,
        string $idDiaFim
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'NRMARCACAOENTRADA' => $nrMarcacaoEntrada,
                'NRMARCACAOSAIDA'   => $nrMarcacaoSaida,
                'DIA'               => $dia,
                'HRENTRADA'         => $hrEntrada,
                'HRSAIDA'           => $hrSaida,
                'DIA_INICIO'        => $idDiaInicio,
                'DIA_FIM'           => $idDiaFim
            ]
        ];

        $result = $this->hcmServices->send('editParMarcacao', $params, $nrOrg);
        $nrMarcacoes = $result['dataset']['editParMarcacao']['nrMarcacoes'];

        return $nrMarcacoes;
    }

    public function saveParMarcacao(
        int $nrOrg,
        int $nrVinculo,
        string $dia,
        string $hrEntrada,
        string $hrSaida,
        string $idDiaInicio,
        string $idDiaFim
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DIA'               => $dia,
                'HRENTRADA'         => $hrEntrada,
                'HRSAIDA'           => $hrSaida,
                'DIA_INICIO'        => $idDiaInicio,
                'DIA_FIM'           => $idDiaFim
            ]
        ];

        $result = $this->hcmServices->send('saveParMarcacao', $params, $nrOrg);
        $nrMarcacoes = $result['dataset']['saveParMarcacao']['nrMarcacoes'];

        return $nrMarcacoes;
    }

    public function saveParesMarcacao(
        int     $nrOrg,
        int     $nrVinculo,
        array   $dia,
        string  $hrEntrada,
        string  $hrSaida,
        string  $idDiaInicio,
        string  $idDiaFim
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DIA'               => $dia,
                'HRENTRADA'         => $hrEntrada,
                'HRSAIDA'           => $hrSaida,
                'DIA_INICIO'        => $idDiaInicio,
                'DIA_FIM'           => $idDiaFim
            ]
        ];

        $result = $this->hcmServices->send('saveParesMarcacao', $params, $nrOrg);
        $nrMarcacoes = $result['dataset']['saveParesMarcacao']['nrMarcacoes'];

        return $nrMarcacoes;
    }

    /**
     * getPeriodoPorTipoVinculo
     * 
     * Busca o período correto, de acordo com o tipo de vínculo. Caso seja passado um vínculo CLT e um
     * nrperiodoapuracao de um estagiário, a função retornará o período de mesmas datas de início e fim porém
     * relativo ao tipo de vínculo CLT
     * 
     * @param string $nrorg
     * @param string $nrvinculom
     * @param string $nrperiodoapuracao
     * @param string $dtinicialapuracao
     * @param string $dtfinalapuracao
     * 
     * @return Array Periodo correto
     * 
     * @throws \Exception
     */
    public function getPeriodoPorTipoVinculo(
        string $nrorg,
        string $nrvinculom,
        string $nrperiodoapuracao,
        string $dtinicialapuracao = null,
        string $dtfinalapuracao = null
    ) : array {
        $vinculo = $this->vinculoRepository->findOneBy(
            array(
                'nrorg' => $nrorg,
                'nrvinculom' => $nrvinculom
            )            
        );

        if (empty($vinculo)) {
            throw new \Exception(self::messages['ERRO_VINCULO_NAO_ENCONTRADO']);
        }

        $getPeriodoParams = array(
            'P_NRORG' => $nrorg,
            'P_NRTIPOVINCULOM' => $vinculo->getNrtipovinculom(),
            'P_DTFINALAPURACAO' => null,
            'P_DTINICIALAPURACAO' => null
        );

        if (!empty($dtinicialapuracao) && !empty($dtfinalapuracao)) {
            $getPeriodoParams['P_DTFINALAPURACAO'] = $dtfinalapuracao;
            $getPeriodoParams['P_DTINICIALAPURACAO'] = $dtinicialapuracao;
        } else {
            $periodo = $this->periodoRepository->findOneBy(array(
                "nrorg"             => $nrorg,
                "nrperiodoapuracao" => $nrperiodoapuracao
            ));

            if (empty($periodo)) {
                throw new \Exception(self::messages['ERRO_PERIODO_NAO_ENCONTRADO']);
            }

            $getPeriodoParams['P_DTFINALAPURACAO'] = $periodo->getDtfinalapuracao()->format('d/m/Y');
            $getPeriodoParams['P_DTINICIALAPURACAO'] = $periodo->getDtinicialapuracao()->format('d/m/Y');
        }

        $periodoValidado = $this->demonstrativoRepository->getPeriodoDemonstrativoPorTipoVinculo($getPeriodoParams);

        if (empty($periodoValidado)) {
            throw new \Exception(self::messages['ERRO_IMPOSSIVEL_ENCONTRAR_PERIODO']);
        }

        return $periodoValidado[0];
    }

    /**
     * getNrPeriodoApuracaoEstruturaPorTipoVinculo
     * 
     * Retorna o nrperiodoapuracão período analisado de acordo com o tipo de vínculo. O processo da busca do
     * período certo é feito em getPeriodoPorTipoVinculo
     * 
     * @param string $nrorg
     * @param string $nrvinculom
     * @param string $nrperiodoapuracao
     * @param string $dtinicialapuracao
     * @param string $dtfinalapuracao
     * 
     * @return string nrperiodoapuracao correto
     * 
     * @throws \Exception
     */
    public function getNrPeriodoApuracaoEstruturaPorTipoVinculo(
        string $nrorg,
        string $nrvinculom,
        string $nrperiodoapuracao,
        string $dtinicialapuracao = null,
        string $dtfinalapuracao = null
    ) : string {

        $periodoValidado = $this->getPeriodoPorTipoVinculo(
            $nrorg,
            $nrvinculom,
            $nrperiodoapuracao,
            $dtinicialapuracao,
            $dtfinalapuracao
        );

        return $periodoValidado['NRPERIODOAPURACAO'];
    }

    public function getEstrutuasGerenciais(
        int $nrOrg,
        int $nrVinculo
    ) : array {

        $params = [
            'P_NRORG'      => $nrOrg,
            'P_NRVINCULOM' => $nrVinculo
        ];

        $periodos = $this->demonstrativoRepository->getEstrutuasGerenciais($params);

        return $periodos;
    }

    public function solicitaAbonoFaltas(
        int    $nrOrg,
        int    $nrVinculo,
        array  $date,
        string $observacao,
        string $file
    ): bool {
        $route = 'solicitaAbonoFaltas';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'    => $nrVinculo,
                'DATE'          => $date,
                'DSOBSERVACAO'  => $observacao,
                'FILE'          => $file,
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }

    public function solicitarCompensacaoHoras(
        int $nrOrg,
        int $nrVinculo,
        string $date
    ): array {

        $hcmServicesRoute = 'solicitarCompensacaoHoras';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DATE'              => $date
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $status = $response['dataset'][$hcmServicesRoute];

        return $status;
    }

    public function solicitaCompensacaoFaltas(
        int     $nrOrg,
        int     $nrVinculo,
        array   $dia
    ): array {
        $route = 'solicitaCompensacaoFaltas';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DATE'              => $dia
            ]
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $status = $result['dataset'][$route];

        return $status;
    }

    public function solicitarHorasExtras(
        int    $nrOrg,
        int    $nrVinculo,
        string $observacao,
        string $date
    ): array {
        $hcmServicesRoute = 'solicitarHorasExtras';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DSOBSERVACAO'      => $observacao,
                'DATE'              => $date
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $status = $response['dataset'][$hcmServicesRoute];

        return $status;
    }

    public function addIsDescandoProp(
        int $nrOrg,
        int $nrVinculo,
        array &$acompanhamento
    ): array {
        $escala = $this->escalaRepository->getEscalaTrabalhoVinculo(
            $nrOrg, $nrVinculo, true
        );

        $diasDescanso = (string) $escala['DESCANSOSEMANAL'];

        $diasSemana = [
            'DOM',
            'SEG',
            'TER',
            'QUAR',
            'QUI',
            'SEX',
            'SAB',
        ];

        foreach ($acompanhamento as &$diaAcompanhamento) {
            $date = $diaAcompanhamento['DIA'];
            $dateTime = DateUtil::getDataDeString($date);
            $diaSemanaNumber = DateUtil::getWeekDay($dateTime);

            $diaSemana = $diasSemana[$diaSemanaNumber];
            $isDescanso = strpos($diasDescanso, $diaSemana) !== false;

            $diaAcompanhamento['ISDESCANSO'] = $isDescanso;
        }

        return $acompanhamento;
    }
}
