CREATE OR REPLACE PROCEDURE GERA_INFADIC_BENEFICIO(
   P_NRSEQUMAPAPU IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_CDIMPOSTO    IN MAPAAPURAC.CDIMPOSTO%TYPE,
   P_CDEMPRESA    IN MAPAAPURAC.CDEMPRESA%TYPE,
   P_CDINSCESTA   IN  MAPAAPURAC.CDINSCESTA%TYPE
)
AS
   V_NRSEQINFADIC    NUMBER;
   V_VRINFADICNF     NUMBER;
   V_VRINFADICTOT    NUMBER;
   V_CODINFADIC      VARCHAR2(10);
   V_DSINFADIC       VARCHAR2(255);
   V_CDBENEC197      VARCHAR2(15);
   V_DSBENEFICIO     VARCHAR2(255);
   V_CDBENEE115      VARCHAR2(15);
   V_VRALIQUOTA      NUMBER;
   V_VRALIQRED       NUMBER;
   V_CDTIPOOPER      VARCHAR2(500);
   V_CDCHAVENOTA     VARCHAR2(15);
   V_NRSEQITMAPR     VARCHAR2(10);
   V_NRSEQDETITMAPA  VARCHAR2(20);
BEGIN

   /******** DELETA OS REGISTROS NA TABELA, DEVIDO A REAPURAO DE MAPA ********/
   DELETE FROM INFADICMAPA
    WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
      AND IDORIGEMINF  = 'B';

   /******** BUSCA O MAIOR SEQUENCIAL DE INFORMAES ADICIONAIS UTILIZADO NO MAPA ********/
   BEGIN
      SELECT MAX(NRSEQINFADIC)
        INTO V_NRSEQINFADIC
        FROM INFADICMAPA
       WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_NRSEQINFADIC := 0;
   END;

   V_CODINFADIC := NULL;
   /******** BUSCA AS NOTAS FISCAIS PRESENTES NO MAPA QUE TENHAM O CDIGO DE BENEFCIO FISCAL PREENCHIDO ********/
   FOR C_NF IN (SELECT IT.CDFILIAL, IT.NRLANCTONF, SUM(ROUND(IT.VRCONTITIMP,2)) AS VRCONTABIL,
                       IT.VRALIQITEM, IT.CDBENEFFISCAL, TB.IDBENEFICIO, TB.DSBENEFICIO
                  FROM TPBENEFICIOFISC TB, ITEMPROIMPOS IT, IMPOSTO IM, NOTAFISCAL NF,
                       (SELECT DISTINCT I.CDFILIAL, I.NRLANCTONF
                          FROM IMPOSTONF I
                         WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU) IP
                 WHERE IP.CDFILIAL      = NF.CDFILIAL
                   AND IP.NRLANCTONF    = NF.NRLANCTONF
                   AND NF.CDFILIAL      = IT.CDFILIAL
                   AND NF.NRLANCTONF    = IT.NRLANCTONF
                   AND IT.CDIMPOSTO     = IM.CDIMPOSTO
                   AND IM.CDIMPOSTONFE  = P_CDIMPOSTO
                   AND NF.IDENTRSAIDOP  = 'S'
                   AND IT.CDBENEFFISCAL = TB.CDBENEFICIO
                   AND NF.CDEMPRESA     = TB.CDEMPRESA
                   AND NF.CDINSCESTA    = TB.CDINSCESTA
                   AND TB.IDBENEFICIO   = 'IOP'
                   AND IT.VRALIQITEM    > 0
                 GROUP BY IT.CDFILIAL, IT.NRLANCTONF, IT.VRALIQITEM, IT.CDBENEFFISCAL, TB.IDBENEFICIO, TB.DSBENEFICIO
                 ORDER BY TB.IDBENEFICIO, IT.CDBENEFFISCAL) LOOP

       IF V_CODINFADIC IS NULL THEN
          V_CODINFADIC := C_NF.CDBENEFFISCAL;
          V_DSINFADIC  := C_NF.DSBENEFICIO;
       END IF;

       /***** CALCULA O VAOR DO BENEFCIO, QUANDO O MESMO FOR ISENO DE OPERAES COM RGO PBLICO *****/
       IF C_NF.IDBENEFICIO = 'IOP' THEN
          V_VRINFADICNF := ROUND((C_NF.VRCONTABIL * 100 / (100 - C_NF.VRALIQITEM)) - C_NF.VRCONTABIL,2);
       END IF;

       /***** QUANDO TROCAR O CDIGO DO BENEFCIO, INSERE OS DADOS NA TABELA REFERENTE A TOTALIZAO DO BENEFCIO ANTERIOR *****/
       IF (V_CODINFADIC <> C_NF.CDBENEFFISCAL) THEN
          IF V_VRINFADICTOT > 0 THEN
             V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
             INSERT INTO INFADICMAPA
                (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
             VALUES
                (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CODINFADIC, V_DSINFADIC, V_VRINFADICTOT);
             V_VRINFADICTOT := NVL(V_VRINFADICNF,0);
             V_CODINFADIC   := NULL;
             V_DSINFADIC    := NULL;
          END IF;
       ELSE
          V_VRINFADICTOT := NVL(V_VRINFADICTOT,0) + V_VRINFADICNF;
       END IF;
   END LOOP;

   /***** AO TERMINAR O LOOP, INSERE O VALOR TOTAL DO LTIMO BENEFCIO *****/
   IF V_VRINFADICTOT > 0 THEN
      V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
      INSERT INTO INFADICMAPA
         (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
      VALUES
         (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CODINFADIC, V_DSINFADIC, V_VRINFADICTOT);
   END IF;

   /******** Deleta da ITMAPARECUP as NFs que vieram da IMPOSTONF que possuem valores informativos,
             gerados de forma automtica pela rotina ********/
   DELETE ITMAPARECUP
    WHERE NRSEQUMAPAPU     = P_NRSEQUMAPAPU
      AND IDTIPOITMAPR     = 'VI'
      AND IDAJUSTEAPURAUTO = 'S';

   /******** Busca o ltimo sequencial na ITMAPARECUP ********/
   V_NRSEQITMAPR := 0;
   BEGIN
      SELECT MAX(NRSEQITMAPR) AS NRSEQITMAPR
        INTO V_NRSEQITMAPR
        FROM ITMAPARECUP
       WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
         AND IDTIPOITMAPR = 'VI';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_NRSEQITMAPR := 0;
   END;

   /*VERIFICA SE EXISTE O CADASTRO PARA GERAO DO BENEFCIO DE ISENO*/
   BEGIN
      SELECT MAX(CDAJUSTENF) AS CDBENEC197, MAX(DSBENEFICIO) AS DSBENEFICIO,
             MAX(CDBENEFICIO) AS CDBENEE115, MAX(VRALIQUOTA) AS VRALIQUOTA,
             MAX(CDTPOPBENEFICIO) AS CDTIPOOPER
        INTO V_CDBENEC197, V_DSBENEFICIO, V_CDBENEE115, V_VRALIQUOTA, V_CDTIPOOPER
        FROM TPBENEFICIOFISC
       WHERE CDEMPRESA   = P_CDEMPRESA
         AND CDINSCESTA  = P_CDINSCESTA
         AND IDBENEFICIO = 'ISE';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDBENEC197  := NULL;
         V_DSBENEFICIO := NULL;
         V_CDBENEE115  := NULL;
         V_VRALIQUOTA  := 0;
         V_CDTIPOOPER  := NULL;
   END;


   IF (V_CDBENEC197 IS NOT NULL) THEN
      V_CDCHAVENOTA  := '';
      V_VRINFADICTOT := 0;
      FOR C_NF IN (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          SUM(ROUND(IT.VRCONTITIMP,2)) AS VRCONTABIL, IT.CDIMPOSTO,
                          IT.NRITPROIMPOS
                     FROM ITEMPROIMPOS IT, IMPOSTO IM,
                          (SELECT DISTINCT I.CDFILIAL, I.NRLANCTONF
                             FROM IMPOSTONF I, NOTAFISCAL N
                            WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                              AND I.CDFISCALNF   = '2'
                              AND I.CDFILIAL     = N.CDFILIAL
                              AND I.NRLANCTONF   = N.NRLANCTONF
                              AND ((INSTR(V_CDTIPOOPER, '_' ||N.IDENTRSAIDOP||N.CDTIPOOPER||'_' ) > 0) OR (V_CDTIPOOPER IS NULL))) IP
                    WHERE IT.CDFILIAL     = IP.CDFILIAL
                      AND IT.NRLANCTONF   = IP.NRLANCTONF
                      AND IT.CDIMPOSTO    = IM.CDIMPOSTO
                      AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                      AND IT.CDFISCALIT   = '2'
                      AND IT.NRITPROIMPOS = '1'
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          IT.CDIMPOSTO, IT.NRITPROIMPOS
                    ORDER BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD) LOOP

          /*CALCULA O VALOR DO ICMS DESONERADO*/
          V_VRINFADICNF := ROUND(C_NF.VRCONTABIL * V_VRALIQUOTA/100,2);
          V_VRINFADICTOT := NVL(V_VRINFADICTOT,0) + V_VRINFADICNF;

          /*QUANDO FOR UMA NOVA NOTA, INSERE OS DADOS DO AJUSTE DA NOTA*/
          IF NVL(V_CDCHAVENOTA,' ') <> C_NF.CDFILIAL||C_NF.NRLANCTONF THEN
             /*INSERE O REGISTRO DE AJUSTE PARA GERAR O C197/D197*/
             V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;
             INSERT INTO ITMAPARECUP
                (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                 DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                 IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA, IDCOMPOEC197, CDCFOPPADRAO,
                 VROUTROSITMAPR, IDAJUSTEAPURAUTO)
             VALUES
                (P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'), V_CDBENEE115, 0, '', '',
                 '', C_NF.NRLANCTONF, C_NF.CDFILIAL, V_CDBENEC197, '', '',
                 'N', 'N', 'N', 'I', '',
                 V_VRINFADICNF, 'S');
             V_CDCHAVENOTA := C_NF.CDFILIAL||C_NF.NRLANCTONF;
          /*QUANDO FOR A MESMA NOTA, ATUALIZA OS VALORES DO AJUSTE*/
          ELSE
             UPDATE ITMAPARECUP
                SET VROUTROSITMAPR = NVL(VROUTROSITMAPR,0) + NVL(V_VRINFADICNF,0)
              WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                AND IDTIPOITMAPR = 'VI'
                AND NRSEQITMAPR  = LPAD(V_NRSEQITMAPR,10,'0');
          END IF;

          /*INSERE OS AJUSTES POR ITENS*/
          NOVO_CODIGO('DETITMAPARECUP',V_NRSEQDETITMAPA);
          INSERT INTO DETITMAPARECUP
             (NRSEQDETITMAPA, NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR,
              CDFILIAL, NRLANCTONF, NRSEQITPROD, CDIMPOSTO, NRITPROIMPOS,
              VROUTROSIMPIT, VROUTALIQ, VROUTBASE)
          VALUES
             (TO_NUMBER(V_NRSEQDETITMAPA), P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'),
              C_NF.CDFILIAL, C_NF.NRLANCTONF, C_NF.NRSEQITPROD, C_NF.CDIMPOSTO, C_NF.NRITPROIMPOS,
              V_VRINFADICNF, V_VRALIQUOTA, C_NF.VRCONTABIL);
      END LOOP;
      IF V_VRINFADICTOT > 0 THEN
         /*INSERE O REGISTRO DE DADOS ADICIONAIS*/
         V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
         INSERT INTO INFADICMAPA
            (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
         VALUES
            (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CDBENEE115, V_DSBENEFICIO, V_VRINFADICTOT);
      END IF;
   END IF;

   /*VERIFICA SE EXISTE O CADASTRO PARA GERAO DO BENEFCIO DE REDUO DA BASE DE CLCULO*/
   BEGIN
      SELECT MAX(CDAJUSTENF) AS CDBENEC197, MAX(DSBENEFICIO) AS DSBENEFICIO,
             MAX(CDBENEFICIO) AS CDBENEE115, MAX(CDTPOPBENEFICIO) AS CDTIPOOPER
        INTO V_CDBENEC197, V_DSBENEFICIO, V_CDBENEE115, V_CDTIPOOPER
        FROM TPBENEFICIOFISC
       WHERE CDEMPRESA   = P_CDEMPRESA
         AND CDINSCESTA  = P_CDINSCESTA
         AND IDBENEFICIO = 'RBC';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDBENEC197  := NULL;
         V_DSBENEFICIO := NULL;
         V_CDBENEE115  := NULL;
         V_VRALIQUOTA  := 0;
         V_CDTIPOOPER  := NULL;
   END;

   IF (V_CDBENEC197 IS NOT NULL) THEN
      V_CDCHAVENOTA  := '';
      V_VRINFADICTOT := 0;
      FOR C_NF IN (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          '' AS CDFILIALVENDA, '' AS CDCAIXA, '' AS NRSEQVENDA,
                          '' AS NRSEQUITVEND, IT.VRALIQITEM+NVL(IT.VRADICALIQFECP,0) AS VRALIQUOTA,
                          IT.VRPERREDBCIT AS VRPERREDUZBC,
                          SUM(ROUND(IT.VRCONTITIMP,2) +
                              ROUND(VALOR_REDUCAO(IT.CDFILIAL, IT.NRLANCTONF,IT.CDSITUTRIBIT, IT.NRSEQITPROD, IT.CDFISCOPERIT, '', IP.CDMODDOCFISC, 'N'),2)) AS VRCONTABIL,
                          IT.CDIMPOSTO, IT.NRITPROIMPOS, IP.IDSISTEMA
                     FROM ITEMPROIMPOS IT, IMPOSTO IM,
                          (SELECT DISTINCT I.CDFILIAL, I.NRLANCTONF, N.IDSISTEMA, N.CDMODDOCFISC
                             FROM IMPOSTONF I, NOTAFISCAL N
                            WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                              AND I.IDIMPOSTONF  = '2'
                              AND I.CDFILIAL     = N.CDFILIAL
                              AND I.NRLANCTONF   = N.NRLANCTONF
                              AND N.IDSISTEMA    <> 'FIS'
                              AND ((INSTR(V_CDTIPOOPER, '_' ||N.IDENTRSAIDOP||N.CDTIPOOPER||'_' ) > 0) OR (V_CDTIPOOPER IS NULL))) IP
                    WHERE IT.CDFILIAL     = IP.CDFILIAL
                      AND IT.NRLANCTONF   = IP.NRLANCTONF
                      AND IT.CDIMPOSTO    = IM.CDIMPOSTO
                      AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                      AND IT.NRITPROIMPOS = '1'
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD, IT.VRALIQITEM+NVL(IT.VRADICALIQFECP,0),
                          IT.VRPERREDBCIT, IT.CDIMPOSTO, IT.NRITPROIMPOS, IP.IDSISTEMA
                    UNION ALL
                   SELECT IP.CDFILIALNOTA AS CDFILIAL, IP.NRLANCTONF, '' AS NRSEQITPROD,
                          IP.CDFILIALVENDA, IP.CDCAIXA, IP.NRSEQVENDA,
                          IT.NRSEQUITVEND, IT.VRPEALPRODIT AS VRALIQUOTA, IT.VRPRBCREDUICMS AS VRPERREDUZBC,
                          SUM(ROUND(IT.VRBASECALCICMS,2)) AS VRCONTABIL,
                          IT.CDIMPOSTO, '' AS NRITPROIMPOS, IP.IDSISTEMA
                     FROM ITVENDAIMPOS IT, IMPOSTO IM,
                          (SELECT DISTINCT R.CDFILIALNOTA, R.NRLANCTONF, R.CDFILIALVENDA, R.CDCAIXA, R.NRSEQVENDA, N.IDSISTEMA
                             FROM IMPOSTONF I, NOTAFISCAL N,  RELACNFCEAPROV R
                            WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                              AND I.IDIMPOSTONF  = '2'
                              AND I.CDFILIAL     = N.CDFILIAL
                              AND I.NRLANCTONF   = N.NRLANCTONF
                              AND N.IDSISTEMA    = 'FIS'
                              AND N.CDFILIAL     = R.CDFILIALNOTA
                              AND N.NRLANCTONF   = R.NRLANCTONF
                              AND ((INSTR(V_CDTIPOOPER, '_' ||N.IDENTRSAIDOP||N.CDTIPOOPER||'_' ) > 0) OR (V_CDTIPOOPER IS NULL))) IP
                    WHERE IT.CDFILIAL     = IP.CDFILIALVENDA
                      AND IT.CDCAIXA      = IP.CDCAIXA
                      AND IT.NRSEQVENDA   = IP.NRSEQVENDA
                      AND IT.CDIMPOSTO    = IM.CDIMPOSTO
                      AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                      AND IM.IDTPIMPOSFIS = 'T'
                      AND IT.VRPRBCREDUICMS > 0
                    GROUP BY IP.CDFILIALNOTA, IP.NRLANCTONF, IP.CDFILIALVENDA, IP.CDCAIXA, IP.NRSEQVENDA,
                          IT.NRSEQUITVEND, IT.VRPEALPRODIT, IT.VRPRBCREDUICMS, IT.CDIMPOSTO, IP.IDSISTEMA
                    ORDER BY 1, 2) LOOP

          /*CALCULA O VALOR DO ICMS DESONERADO*/
          /*Valor do ICMS desonerado = Preo na Nota Fiscal * (1 - (Alquota * (1 - Percentual de reduo da BC))) / (1 - Alquota) - Preo na Nota Fiscal*/
          V_VRINFADICNF  := ROUND(C_NF.VRCONTABIL * (1 - ((C_NF.VRALIQUOTA/100) * (1 - (C_NF.VRPERREDUZBC/100)))) / (1 - (C_NF.VRALIQUOTA/100)) - C_NF.VRCONTABIL,2);
          V_VRINFADICTOT := NVL(V_VRINFADICTOT,0) + V_VRINFADICNF;

          /*QUANDO FOR UMA NOVA NOTA, INSERE OS DADOS DO AJUSTE DA NOTA*/
          IF NVL(V_CDCHAVENOTA,' ') <> C_NF.CDFILIAL||C_NF.NRLANCTONF THEN
             /*INSERE O REGISTRO DE AJUSTE PARA GERAR O C197/D197*/
             V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;
             INSERT INTO ITMAPARECUP
                (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                 DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                 IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA, IDCOMPOEC197, CDCFOPPADRAO,
                 VROUTROSITMAPR, IDAJUSTEAPURAUTO)
             VALUES
                (P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'), V_CDBENEE115, 0, '', '',
                 '', C_NF.NRLANCTONF, C_NF.CDFILIAL, V_CDBENEC197, '', '',
                 'N', 'N', 'N', 'I', '',
                 V_VRINFADICNF, 'S');
             V_CDCHAVENOTA := C_NF.CDFILIAL||C_NF.NRLANCTONF;
          /*QUANDO FOR A MESMA NOTA, ATUALIZA OS VALORES DO AJUSTE*/
          ELSE
             UPDATE ITMAPARECUP
                SET VROUTROSITMAPR = NVL(VROUTROSITMAPR,0) + NVL(V_VRINFADICNF,0)
              WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                AND IDTIPOITMAPR = 'VI'
                AND NRSEQITMAPR  = LPAD(V_NRSEQITMAPR,10,'0');
          END IF;

          /*INSERE OS AJUSTES POR ITENS*/
          NOVO_CODIGO('DETITMAPARECUP',V_NRSEQDETITMAPA);
          INSERT INTO DETITMAPARECUP
             (NRSEQDETITMAPA, NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR,
              CDFILIAL, NRLANCTONF, NRSEQITPROD, CDIMPOSTO, NRITPROIMPOS,
              VROUTROSIMPIT, VROUTALIQ, VROUTBASE, CDFILIALVND, CDCAIXA,
              NRSEQVENDA, NRSEQUITVEND)
          VALUES
             (TO_NUMBER(V_NRSEQDETITMAPA), P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'),
              C_NF.CDFILIAL, C_NF.NRLANCTONF, C_NF.NRSEQITPROD, C_NF.CDIMPOSTO, C_NF.NRITPROIMPOS,
              V_VRINFADICNF, V_VRALIQUOTA, C_NF.VRCONTABIL, C_NF.CDFILIALVENDA, C_NF.CDCAIXA,
              C_NF.NRSEQVENDA, C_NF.NRSEQUITVEND);
      END LOOP;
      IF V_VRINFADICTOT > 0 THEN
         /*INSERE O REGISTRO DE DADOS ADICIONAIS*/
         V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
         INSERT INTO INFADICMAPA
            (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
         VALUES
            (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CDBENEE115, V_DSBENEFICIO, V_VRINFADICTOT);
      END IF;
   END IF;

   /*VERIFICA SE EXISTE O CADASTRO PARA GERAO DO BENEFCIO DE REDUO DA ALQUOTA*/
   BEGIN
      SELECT MAX(CDAJUSTENF) AS CDBENEC197, MAX(DSBENEFICIO) AS DSBENEFICIO,
             MAX(CDBENEFICIO) AS CDBENEE115, MAX(VRALIQUOTA) AS VRALIQUOTA,
             MAX(VRALIQRED) AS VRALIQRED, MAX(CDTPOPBENEFICIO) AS CDTIPOOPER
        INTO V_CDBENEC197, V_DSBENEFICIO, V_CDBENEE115, V_VRALIQUOTA, V_VRALIQRED, V_CDTIPOOPER
        FROM TPBENEFICIOFISC
       WHERE CDEMPRESA   = P_CDEMPRESA
         AND CDINSCESTA  = P_CDINSCESTA
         AND IDBENEFICIO = 'RAL';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDBENEC197  := NULL;
         V_DSBENEFICIO := NULL;
         V_CDBENEE115  := NULL;
         V_VRALIQUOTA  := 0;
         V_VRALIQRED   := 0;
         V_CDTIPOOPER  := NULL;
   END;

   IF (V_CDBENEC197 IS NOT NULL) THEN
      V_CDCHAVENOTA  := '';
      V_VRINFADICTOT := 0;
      FOR C_NF IN (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          SUM(ROUND(IT.VRBASECALCIT,2)) AS VRBASECALC, IT.CDIMPOSTO,
                          IT.NRITPROIMPOS
                     FROM ITEMPROIMPOS IT, IMPOSTO IM,
                          (SELECT DISTINCT I.CDFILIAL, I.NRLANCTONF
                             FROM IMPOSTONF I, NOTAFISCAL N
                            WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                              AND I.CDFISCALNF   = '1'
                              AND I.VRALIQNF     = V_VRALIQRED
                              AND I.CDFILIAL     = N.CDFILIAL
                              AND I.NRLANCTONF   = N.NRLANCTONF
                              AND ((INSTR(V_CDTIPOOPER, '_' ||N.IDENTRSAIDOP||N.CDTIPOOPER||'_' ) > 0) OR (V_CDTIPOOPER IS NULL))) IP
                    WHERE IT.CDFILIAL     = IP.CDFILIAL
                      AND IT.NRLANCTONF   = IP.NRLANCTONF
                      AND IT.CDIMPOSTO    = IM.CDIMPOSTO
                      AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                      AND IT.CDFISCALIT   = '1'
                      AND IT.VRALIQITEM   = V_VRALIQRED
                      AND IT.NRITPROIMPOS = '1'
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          IT.CDIMPOSTO, IT.NRITPROIMPOS
                    ORDER BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD) LOOP

          /*CALCULA O VALOR DO ICMS DESONERADO*/
          V_VRINFADICNF  := ROUND(C_NF.VRBASECALC * (V_VRALIQUOTA-V_VRALIQRED)/100,2);
          V_VRINFADICTOT := NVL(V_VRINFADICTOT,0) + V_VRINFADICNF;

          /*QUANDO FOR UMA NOVA NOTA, INSERE OS DADOS DO AJUSTE DA NOTA*/
          IF NVL(V_CDCHAVENOTA,' ') <> C_NF.CDFILIAL||C_NF.NRLANCTONF THEN
             /*INSERE O REGISTRO DE AJUSTE PARA GERAR O C197/D197*/
             V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;
             INSERT INTO ITMAPARECUP
                (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                 DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                 IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA, IDCOMPOEC197, CDCFOPPADRAO,
                 VROUTROSITMAPR, IDAJUSTEAPURAUTO)
             VALUES
                (P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'), V_CDBENEE115, 0, '', '',
                 '', C_NF.NRLANCTONF, C_NF.CDFILIAL, V_CDBENEC197, '', '',
                 'N', 'N', 'N', 'I', '',
                 V_VRINFADICNF, 'S');
             V_CDCHAVENOTA := C_NF.CDFILIAL||C_NF.NRLANCTONF;
          /*QUANDO FOR A MESMA NOTA, ATUALIZA OS VALORES DO AJUSTE*/
          ELSE
             UPDATE ITMAPARECUP
                SET VROUTROSITMAPR = NVL(VROUTROSITMAPR,0) + NVL(V_VRINFADICNF,0)
              WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                AND IDTIPOITMAPR = 'VI'
                AND NRSEQITMAPR  = LPAD(V_NRSEQITMAPR,10,'0');
          END IF;

          /*INSERE OS AJUSTES POR ITENS*/
          NOVO_CODIGO('DETITMAPARECUP',V_NRSEQDETITMAPA);
          INSERT INTO DETITMAPARECUP
             (NRSEQDETITMAPA, NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR,
              CDFILIAL, NRLANCTONF, NRSEQITPROD, CDIMPOSTO, NRITPROIMPOS,
              VROUTROSIMPIT, VROUTALIQ, VROUTBASE)
          VALUES
             (TO_NUMBER(V_NRSEQDETITMAPA), P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'),
              C_NF.CDFILIAL, C_NF.NRLANCTONF, C_NF.NRSEQITPROD, C_NF.CDIMPOSTO, C_NF.NRITPROIMPOS,
              V_VRINFADICNF, (V_VRALIQUOTA-V_VRALIQRED), C_NF.VRBASECALC);
      END LOOP;
      IF V_VRINFADICTOT > 0 THEN
         /*INSERE O REGISTRO DE DADOS ADICIONAIS*/
         V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
         INSERT INTO INFADICMAPA
            (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
         VALUES
            (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CDBENEE115, V_DSBENEFICIO, V_VRINFADICTOT);
      END IF;
   END IF;

   /*VERIFICA SE EXISTE O CADASTRO PARA GERAO DO BENEFCIO DE NO INCIDNCIA*/
   BEGIN
      SELECT MAX(CDAJUSTENF) AS CDBENEC197, MAX(DSBENEFICIO) AS DSBENEFICIO,
             MAX(CDBENEFICIO) AS CDBENEE115, MAX(VRALIQUOTA) AS VRALIQUOTA,
             MAX(CDTPOPBENEFICIO) AS CDTIPOOPER
        INTO V_CDBENEC197, V_DSBENEFICIO, V_CDBENEE115, V_VRALIQUOTA, V_CDTIPOOPER
        FROM TPBENEFICIOFISC
       WHERE CDEMPRESA   = P_CDEMPRESA
         AND CDINSCESTA  = P_CDINSCESTA
         AND IDBENEFICIO = 'NIN';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDBENEC197  := NULL;
         V_DSBENEFICIO := NULL;
         V_CDBENEE115  := NULL;
         V_VRALIQUOTA  := 0;
         V_CDTIPOOPER  := NULL;
   END;

   IF (V_CDBENEC197 IS NOT NULL) THEN
      V_CDCHAVENOTA  := '';
      V_VRINFADICTOT := 0;
      FOR C_NF IN (SELECT IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          SUM(ROUND(IT.VRCONTITIMP,2)) AS VRCONTABIL, IT.CDIMPOSTO,
                          IT.NRITPROIMPOS
                     FROM ITEMPROIMPOS IT, IMPOSTO IM,
                          (SELECT DISTINCT I.CDFILIAL, I.NRLANCTONF
                             FROM IMPOSTONF I, NOTAFISCAL N
                            WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                              AND I.CDFISCALNF   = '3'
                              AND I.CDFILIAL     = N.CDFILIAL
                              AND I.NRLANCTONF   = N.NRLANCTONF
                              AND ((INSTR(V_CDTIPOOPER, '_' ||N.IDENTRSAIDOP||N.CDTIPOOPER||'_' ) > 0) OR (V_CDTIPOOPER IS NULL))) IP
                    WHERE IT.CDFILIAL     = IP.CDFILIAL
                      AND IT.NRLANCTONF   = IP.NRLANCTONF
                      AND IT.CDIMPOSTO    = IM.CDIMPOSTO
                      AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                      AND IT.CDFISCALIT   = '3'
                      AND IT.NRITPROIMPOS = '1'
                    GROUP BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD,
                          IT.CDIMPOSTO, IT.NRITPROIMPOS
                    ORDER BY IT.CDFILIAL, IT.NRLANCTONF, IT.NRSEQITPROD) LOOP

          /*CALCULA O VALOR DO ICMS DESONERADO*/
          V_VRINFADICNF  := ROUND(C_NF.VRCONTABIL * V_VRALIQUOTA/100,2);
          V_VRINFADICTOT := NVL(V_VRINFADICTOT,0) + V_VRINFADICNF;

          /*QUANDO FOR UMA NOVA NOTA, INSERE OS DADOS DO AJUSTE DA NOTA*/
          IF NVL(V_CDCHAVENOTA,' ') <> C_NF.CDFILIAL||C_NF.NRLANCTONF THEN
             /*INSERE O REGISTRO DE AJUSTE PARA GERAR O C197/D197*/
             V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;
             INSERT INTO ITMAPARECUP
                (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                 DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                 IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA, IDCOMPOEC197, CDCFOPPADRAO,
                 VROUTROSITMAPR, IDAJUSTEAPURAUTO)
             VALUES
                (P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'), V_CDBENEE115, 0, '', '',
                 '', C_NF.NRLANCTONF, C_NF.CDFILIAL, V_CDBENEC197, '', '',
                 'N', 'N', 'N', 'I', '',
                 V_VRINFADICNF, 'S');
             V_CDCHAVENOTA := C_NF.CDFILIAL||C_NF.NRLANCTONF;
          /*QUANDO FOR A MESMA NOTA, ATUALIZA OS VALORES DO AJUSTE*/
          ELSE
             UPDATE ITMAPARECUP
                SET VROUTROSITMAPR = NVL(VROUTROSITMAPR,0) + NVL(V_VRINFADICNF,0)
              WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                AND IDTIPOITMAPR = 'VI'
                AND NRSEQITMAPR  = LPAD(V_NRSEQITMAPR,10,'0');
          END IF;

          /*INSERE OS AJUSTES POR ITENS*/
          NOVO_CODIGO('DETITMAPARECUP',V_NRSEQDETITMAPA);
          INSERT INTO DETITMAPARECUP
             (NRSEQDETITMAPA, NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR,
              CDFILIAL, NRLANCTONF, NRSEQITPROD, CDIMPOSTO, NRITPROIMPOS,
              VROUTROSIMPIT, VROUTALIQ, VROUTBASE)
          VALUES
             (TO_NUMBER(V_NRSEQDETITMAPA), P_NRSEQUMAPAPU, 'VI', LPAD(V_NRSEQITMAPR,10,'0'),
              C_NF.CDFILIAL, C_NF.NRLANCTONF, C_NF.NRSEQITPROD, C_NF.CDIMPOSTO, C_NF.NRITPROIMPOS,
              V_VRINFADICNF, V_VRALIQUOTA, C_NF.VRCONTABIL);
      END LOOP;
      IF V_VRINFADICTOT > 0 THEN
         /*INSERE O REGISTRO DE DADOS ADICIONAIS*/
         V_NRSEQINFADIC := NVL(V_NRSEQINFADIC,0) + 1;
         INSERT INTO INFADICMAPA
            (NRSEQUMAPAPU, NRSEQINFADIC, IDORIGEMINF, CDINFADICMAPA, DSINFADICMAPA, VRINFADICMAPA)
         VALUES
            (P_NRSEQUMAPAPU, LPAD(V_NRSEQINFADIC,5,'0'), 'B', V_CDBENEE115, V_DSBENEFICIO, V_VRINFADICTOT);
      END IF;
   END IF;
END;